﻿using System;
using MVR.FileManagementSecure;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using SimpleJSON;

namespace ky1001
{
    public class PresetLoader : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "PresetLoader";
        public string pluginVersion = "8.0";
        public string pluginDate = "2022-09-20";
        public string pluginDescription = "";
        #endregion

        private Atom _targetAtom;

        private const string _clothingPresetPath = "Custom/Atom/Person/Clothing";
        private const string _hairPresetPath = "Custom/Atom/Person/Hair";
        private const string _posePresetPath = "Custom/Atom/Person/Pose";
        private const string _skinPresetPath = "Custom/Atom/Person/Skin";
        private const string _filePrefs = "AddonPackagesFilePrefs/";
        private const int _jsonstringLimit = 15000;
        private const float _currentPathHeight = 100f;
        private const float _presetPathHeight = 165f;

        private JSONStorableBool _enableClothing;
        private JSONStorableBool _enableHair;
        private JSONStorableBool _enablePose;
        private JSONStorableBool _enableSkin;
        private JSONStorableBool _enablePreset;
        private JSONStorableBool _enableRnd;
        private JSONStorableBool _enableAutoLoad;
        private JSONStorableBool _enableSubfolder;
        private JSONStorableBool _onlyFavClothing;
        private JSONStorableBool _onlyFavHair;
        private JSONStorableBool _onlyFavPose;
        private JSONStorableBool _onlyFavSkin;
        private JSONStorableBool _onlyFav;
        private JSONStorableBool _enablePositionAdjust;
        private JSONStorableString _clothingFilesInFolder;
        private JSONStorableString _hairFilesInFolder;
        private JSONStorableString _poseFilesInFolder;
        private JSONStorableString _skinFilesInFolder;
        private JSONStorableString _filesInFolder;
        private JSONStorableString _currentClothing;
        private JSONStorableString _currentHair;
        private JSONStorableString _currentPose;
        private JSONStorableString _currentSkin;
        private JSONStorableString _current;
        private string _presetClothingPath;
        private string _presetHairPath;
        private string _presetPosePath;
        private string _presetSkinPath;
        private string _favClothingPath;
        private string _favHairPath;
        private string _favPosePath;
        private string _favSkinPath;
        private JSONStorableString _savedClothingPath;
        private JSONStorableString _savedHairPath;
        private JSONStorableString _savedPosePath;
        private JSONStorableString _savedSkinPath;
        private JSONStorableStringChooser _selectCount;
        private List<string> _clothingList = new List<string>();
        private List<string> _hairList = new List<string>();
        private List<string> _poseList = new List<string>();
        private List<string> _skinList = new List<string>();
        private List<string> _presetClothingList = new List<string>();
        private List<string> _presetHairList = new List<string>();
        private List<string> _presetPoseList = new List<string>();
        private List<string> _presetSkinList = new List<string>();
        private List<string> _favClothingList = new List<string>();
        private List<string> _favHairList = new List<string>();
        private List<string> _favPoseList = new List<string>();
        private List<string> _favSkinList = new List<string>();
        private List<string> _presetList = new List<string>();
        private List<string> _favList = new List<string>();
        private List<string> _femaleList = new List<string>();
        private List<string> _activeClothingList = new List<string>();
        private string _presetPath;
        private string _varFileName;
        private string _varFullName;
        private string _varFullPath;
        private int _previousClothingNumber = 0;
        private int _previousHairNumber = 0;
        private int _previousPoseNumber = 0;
        private int _previousSkinNumber = 0;
        private string _previousSelectCount;
        private bool _loopStart = false;
        private float _waitCount;
        private bool _isFemale;
        private string _posePresetFile;
        private string _skinPresetFile;
        private string _loadData;
        private JSONNode _jSONNode;
        private JSONArray _asArray;
        private JSONClass _asObject;
        private int _startTime;
        private int _currentTime;
        private int _selectTime;
        private string _id;
        private Vector3 _personPosi;
        private Vector3 _loadedPosi;
        private bool _enablePush;
        private bool _isPosition;
        private float _min;
        private Vector3 _control;
        private Vector3 _hipControl;
        private Vector3 _posi;
        private Vector3 _rota;
        private Dictionary<string, Vector3> _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
        private Dictionary<string, Vector3> _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();
        private Vector3 _rootPosi;
        private Vector3 _rootRota;

        private readonly string[] _female = new string[]
        {
            "Candy",
            "Evey",
            "Janie",
            "Kayla",
            "Lexi",
            "Maria",
            "Mia",
            "Simone",
            "Tara",
            "Tina"
        };

        private readonly string[] _rigidbodyNames = new string[]
        {
            "hip",
            "pelvis",

            "rThigh",
            "rShin",
            "rFoot",
            "rToe",

            "lThigh",
            "lShin",
            "lFoot",
            "lToe",

            "abdomen",
            "abdomen2",

            "chest",

            "lPectoral",
            "rPectoral",

            "rCollar",
            "rShldr",
            "rForeArm",
            "rHand",

            "lCollar",
            "lShldr",
            "lForeArm",
            "lHand",

            "neck",
            "head"
        };

        public override void Init()
        {
            try
            {
                _targetAtom = containingAtom;

                if (_targetAtom.type != "Person")
                {
                    SuperController.LogError("Please attach this plugin to Person");
                    return;
                }

                var loadPresetButton = CreateButton("Load Presets");
                loadPresetButton.button.onClick.AddListener(LoadPreset);
                JSONStorableAction LoadPresetAct = new JSONStorableAction("Load Presets", LoadPreset);
                _enablePush = true;

                _enableAutoLoad = new JSONStorableBool("Enable Auto Load", false);
                CreateToggle(_enableAutoLoad, true);


                _enableRnd = new JSONStorableBool("Enable Random Load", true);
                var enableRnd = CreateToggle(_enableRnd);
                enableRnd.toggle.onValueChanged.AddListener(delegate { ResetCount(); });

                var sp = CreateSpacer();
                sp.height = 14f;

                _enableSubfolder = new JSONStorableBool("Include Subfolder", true);
                CreateToggle(_enableSubfolder);

                List<string> waitCount = new List<string>();
                waitCount.Add("5");
                waitCount.Add("15");
                waitCount.Add("30");
                waitCount.Add("60");
                waitCount.Add("120");
                waitCount.Add("180");

                _selectCount = new JSONStorableStringChooser("Interval", waitCount, "30", "Interval [sec]");
                var sc = CreatePopup(_selectCount, true);

                _enablePositionAdjust = new JSONStorableBool("Enable Position Adjustment", true);
                CreateToggle(_enablePositionAdjust, true);

                var presetName = "Clothing";
                MakeUi(presetName, SetClothingPath);
                _enableClothing = _enablePreset;
                _currentClothing = _current;
                _clothingFilesInFolder = _filesInFolder;
                _onlyFavClothing = _onlyFav;
                JSONStorableAction SetClothingPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetClothingPathAct);

                presetName = "Hair";
                MakeUi(presetName, SetHairPath);
                _enableHair = _enablePreset;
                _currentHair = _current;
                _hairFilesInFolder = _filesInFolder;
                _onlyFavHair = _onlyFav;
                JSONStorableAction SetHairPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetHairPathAct);

                presetName = "Pose";
                MakeUi(presetName, SetPosePath);
                _enablePose = _enablePreset;
                _currentPose = _current;
                _poseFilesInFolder = _filesInFolder;
                _onlyFavPose = _onlyFav;
                JSONStorableAction SetPosePresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetPosePathAct);

                presetName = "Skin";
                MakeUi(presetName, SetSkinPath);
                _enableSkin = _enablePreset;
                _currentSkin = _current;
                _skinFilesInFolder = _filesInFolder;
                _onlyFavSkin = _onlyFav;
                JSONStorableAction SetSkinPresetPathAct = new JSONStorableAction("Set " + presetName + " Presets Path", SetSkinPathAct);

                _savedClothingPath = new JSONStorableString("Clothing Presets Path", "");
                _savedHairPath = new JSONStorableString("Hair Presets Path", "");
                _savedPosePath = new JSONStorableString("Pose Preset Path", "");
                _savedSkinPath = new JSONStorableString("Skin Preset Path", "");

                _femaleList = new List<string>(_female);
                _selectTime = int.Parse(_selectCount.val);

                RegisterAction(LoadPresetAct);
                RegisterAction(SetClothingPresetPathAct);
                RegisterAction(SetHairPresetPathAct);
                RegisterAction(SetPosePresetPathAct);
                RegisterAction(SetSkinPresetPathAct);
                RegisterStringChooser(_selectCount);
                RegisterBool(_enableAutoLoad);
                RegisterBool(_enableSubfolder);
                RegisterBool(_enableRnd);
                RegisterBool(_enableClothing);
                RegisterBool(_enableHair);
                RegisterBool(_enablePose);
                RegisterBool(_enableSkin);
                RegisterBool(_onlyFavClothing);
                RegisterBool(_onlyFavHair);
                RegisterBool(_onlyFavPose);
                RegisterBool(_onlyFavSkin);
                RegisterBool(_enablePositionAdjust);
                RegisterString(_savedClothingPath);
                RegisterString(_savedHairPath);
                RegisterString(_savedPosePath);
                RegisterString(_savedSkinPath);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        public void MakeUi(string presetName, UnityAction action)
        {
            var setPresetPath = CreateButton("Set " + presetName + " Presets Path");
            setPresetPath.button.onClick.AddListener(action);

            _enablePreset = new JSONStorableBool("Enable " + presetName + " Presets", false);
            CreateToggle(_enablePreset);
            _onlyFav = new JSONStorableBool("Favorite " + presetName + " Only", false);
            var onlyFavToggle = CreateToggle(_onlyFav, true);
            onlyFavToggle.toggle.onValueChanged.AddListener(delegate { ChangePath(presetName); });

            _current = new JSONStorableString("", "");
            var current = CreateTextField(_current);
            current.height = _currentPathHeight;
            _filesInFolder = new JSONStorableString("", "");
            var path = CreateTextField(_filesInFolder, true);
            path.height = _presetPathHeight;
        }

        public void ChangePath(string presetName)
        {
            if (presetName == "Clothing")
            {
                if (_onlyFavClothing.val)
                {
                    _clothingFilesInFolder.val = _favClothingPath;
                    _clothingList = new List<string>(_favClothingList);
                }
                else
                {
                    _clothingFilesInFolder.val = _presetClothingPath;
                    _clothingList = new List<string>(_presetClothingList);
                }
                _currentClothing.val = "";
                _previousClothingNumber = 0;
            }
            else if (presetName == "Hair")
            {
                if (_onlyFavHair.val)
                {
                    _hairFilesInFolder.val = _favHairPath;
                    _hairList = new List<string>(_favHairList);
                }
                else
                {
                    _hairFilesInFolder.val = _presetHairPath;
                    _hairList = new List<string>(_presetHairList);
                }
                _currentHair.val = "";
                _previousHairNumber = 0;
            }
            else if (presetName == "Pose")
            {
                if (_onlyFavPose.val)
                {
                    _poseFilesInFolder.val = _favPosePath;
                    _poseList = new List<string>(_favPoseList);
                }
                else
                {
                    _poseFilesInFolder.val = _presetPosePath;
                    _poseList = new List<string>(_presetPoseList);
                }
                _currentPose.val = "";
                _previousPoseNumber = 0;
            }
            else
            {
                if (_onlyFavSkin.val)
                {
                    _skinFilesInFolder.val = _favSkinPath;
                    _skinList = new List<string>(_favSkinList);
                }
                else
                {
                    _skinFilesInFolder.val = _presetSkinPath;
                    _skinList = new List<string>(_presetSkinList);
                }
                _currentSkin.val = "";
                _previousSkinNumber = 0;
            }
        }

        public void ResetCount()
        {
            _previousClothingNumber = 0;
            _previousHairNumber = 0;
            _previousPoseNumber = 0;
            _previousSkinNumber = 0;
        }

        public void LoadPreset()
        {
            if (!_enablePush)
            {
                return;
            }
            else
            {
                _enablePush = false;
            }

            int number;
            if (_enableAutoLoad.val)
            {
                if (_loopStart)
                {
                    _waitCount = _selectTime;
                }
                else
                {
                    _loopStart = true;
                    return;
                }
            }
            else
            {
                _loopStart = false;
                _waitCount = 0;
            }

            //_targetAtom.collisionEnabledJSON.val = false;

            if (_enableClothing.val)
            {
                if (_clothingList.Count != 0)
                {
                    if (_clothingList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _clothingList.Count);
                            }
                            while (_previousClothingNumber == number);
                            _previousClothingNumber = number;
                        }
                        else
                        {
                            number = _previousClothingNumber;
                            _previousClothingNumber = number + 1;
                            if (_previousClothingNumber >= _clothingList.Count)
                            {
                                _previousClothingNumber = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _clothingList[number];
                    _targetAtom.GetStorableByID("ClothingPresets").CallAction("LoadPreset");
                    _currentClothing.val = _clothingList[number];
                }
            }

            if (_enableHair.val)
            {
                if (_hairList.Count != 0)
                {
                    if (_hairList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _hairList.Count);
                            }
                            while (_previousHairNumber == number);
                            _previousHairNumber = number;
                        }
                        else
                        {
                            number = _previousHairNumber;
                            _previousHairNumber = number + 1;
                            if (_previousHairNumber >= _hairList.Count)
                            {
                                _previousHairNumber = 0;
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _hairList[number];
                    _targetAtom.GetStorableByID("HairPresets").CallAction("LoadPreset");
                    _currentHair.val = _hairList[number];
                }
            }

            if (_enablePose.val)
            {
                if (_poseList.Count != 0)
                {
                    if (_poseList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _poseList.Count);
                            }
                            while (_previousPoseNumber == number);
                            _previousPoseNumber = number;
                        }
                        else
                        {
                            number = _previousPoseNumber;
                            _previousPoseNumber = number + 1;
                            if (_previousPoseNumber >= _poseList.Count)
                            {
                                _previousPoseNumber = 0;
                            }
                        }
                    }

                    if (_poseList[number].Contains(":"))
                    {
                        var posePresetSplit = _poseList[number].Split(':');
                        _posePresetFile = posePresetSplit[0] + ":/" + _posePresetPath + "/" + posePresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _posePresetFile = _posePresetPath + "/" + _poseList[number] + ".vap";
                    }

                    int iPosi = _posePresetFile.LastIndexOf("/");
                    _posePresetFile = _posePresetFile.Insert(iPosi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_posePresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    _isPosition = false;
                    _control = Vector3.zero;

                    _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
                    _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
                    _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
                    _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "control" && _asObject["position"] != null)
                        {
                            _isPosition = true;
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _control = _posi;
                        }
                        else if (_id == "hipControl" && _asObject["localPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["localPosition"][0]);
                            _posi.y = float.Parse(_asObject["localPosition"][1]);
                            _posi.z = float.Parse(_asObject["localPosition"][2]);
                            _hipControl = _posi;
                        }
                        else if (_id == "hip" && _asObject["rootPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["rootPosition"][0]);
                            _posi.y = float.Parse(_asObject["rootPosition"][1]);
                            _posi.z = float.Parse(_asObject["rootPosition"][2]);
                            _rota.x = float.Parse(_asObject["rootRotation"][0]);
                            _rota.y = float.Parse(_asObject["rootRotation"][1]);
                            _rota.z = float.Parse(_asObject["rootRotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);
                            _rigidbodyPosiWorld[_id] = _posi;
                            _rigidbodyRotaWorld[_id] = Quaternion.Euler(_rota);
                            _rootPosi = _posi;
                            _rootRota = _rota;
                        }
                        else if (Array.IndexOf(_rigidbodyNames, _id) >= 0 && _asObject["position"] != null)
                        {
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _rota.x = float.Parse(_asObject["rotation"][0]);
                            _rota.y = float.Parse(_asObject["rotation"][1]);
                            _rota.z = float.Parse(_asObject["rotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);

                            GameObject parent = _targetAtom.GetStorableByID(_id).transform.parent.gameObject;

                            GameObject parentRigid = new GameObject();
                            GameObject currentRigid = new GameObject();

                            parentRigid.transform.position = _rigidbodyPosiWorld[parent.name];
                            parentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name];
                            currentRigid.transform.position = parentRigid.transform.TransformPoint(_posi);
                            currentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name] * _rigidbodyRotaLocal[_id];
                            _rigidbodyPosiWorld[_id] = currentRigid.transform.position;
                            _rigidbodyRotaWorld[_id] = currentRigid.transform.rotation;
                        }
                    }

                    var minY = 2f;
                    var minName = "hip";
                    foreach (var item in _rigidbodyNames)
                    {
                        if (_rigidbodyPosiWorld[item] != null)
                        {
                            var thisY = _rigidbodyPosiWorld[item].y;
                            if (minY > thisY)
                            {
                                minY = thisY;
                                minName = item;
                            }
                        }
                    }

                    _min = -_rootPosi.y + minY + _hipControl.y;

                    _personPosi = _targetAtom.GetStorableByID("control").transform.position;
                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        _activeClothingList = new List<string>();
                        _targetAtom.tempDisableRender = !_targetAtom.tempDisableRender;
                        var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                        foreach (var item in allGeometry)
                        {
                            if (item.Contains("clothing:"))
                            {
                                if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                                {
                                    _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                                    _activeClothingList.Add(item);
                                }
                            }
                        }
                    }
                    _targetAtom.GetStorableByID("PosePresets").GetStringJSONParam("presetName").val = _poseList[number];
                    _targetAtom.GetStorableByID("PosePresets").CallAction("LoadPreset");
                    _loadedPosi = _targetAtom.GetStorableByID("control").transform.position;
                    _currentPose.val = _poseList[number];

                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        StartCoroutine(MoveRoot());
                    }
                }
            }

            if (_enableSkin.val)
            {
                if (_skinList.Count != 0)
                {
                    if (_skinList.Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_enableRnd.val)
                        {
                            do
                            {
                                number = UnityEngine.Random.Range(0, _skinList.Count);
                            }
                            while (_previousSkinNumber == number);
                            _previousSkinNumber = number;
                        }
                        else
                        {
                            number = _previousSkinNumber;
                            _previousSkinNumber = number + 1;
                            if (_previousSkinNumber >= _skinList.Count)
                            {
                                _previousSkinNumber = 0;
                            }
                        }
                    }

                    var characterName = _targetAtom.GetStorableByID("geometry").GetStringChooserJSONParam("characterSelection").val;
                    if (_femaleList.Contains(characterName) || characterName.Contains("Female"))
                    {
                        _isFemale = true;
                    }
                    else
                    {
                        _isFemale = false;
                    }

                    if (_skinList[number].Contains(":"))
                    {
                        var skinPresetSplit = _skinList[number].Split(':');
                        _skinPresetFile = skinPresetSplit[0] + ":/" + _skinPresetPath + "/" + skinPresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _skinPresetFile = _skinPresetPath + "/" + _skinList[number] + ".vap";
                    }

                    int posi = _skinPresetFile.LastIndexOf("/");
                    _skinPresetFile = _skinPresetFile.Insert(posi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_skinPresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "geometry" && _asObject["character"] != null)
                        {
                            characterName = _asObject["character"];
                        }
                    }

                    bool _isFemaleInVap = _femaleList.Contains(characterName) || characterName.Contains("Female");
                    if (_isFemale == _isFemaleInVap)
                    {
                        _targetAtom.GetStorableByID("SkinPresets").GetStringJSONParam("presetName").val = _skinList[number];
                        _targetAtom.GetStorableByID("SkinPresets").CallAction("LoadPreset");
                        _currentSkin.val = _skinList[number];
                    }
                    else
                    {
                        _currentSkin.val = "";
                    }
                }
            }
            _enablePush = true;

            //Invoke("CollisionOn", 1f);
        }

        public void CollisionOn()
        {
            //_targetAtom.collisionEnabledJSON.val = true;
            _enablePush = true;
        }

        public void SetClothingPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_clothingPresetPath);
            _presetPath = _clothingPresetPath;
            _onlyFav = _onlyFavClothing;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetClothingPathAct()
        {
            _presetPath = _clothingPresetPath;
            _onlyFav = _onlyFavClothing;
            LoadPresetsList(_savedClothingPath.val);
        }

        public void SetHairPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_hairPresetPath);
            _presetPath = _hairPresetPath;
            _onlyFav = _onlyFavHair;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetHairPathAct()
        {
            _presetPath = _hairPresetPath;
            _onlyFav = _onlyFavHair;
            LoadPresetsList(_savedHairPath.val);
        }

        public void SetPosePath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_posePresetPath);
            _presetPath = _posePresetPath;
            _onlyFav = _onlyFavPose;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetPosePathAct()
        {
            _presetPath = _posePresetPath;
            _onlyFav = _onlyFavPose;
            LoadPresetsList(_savedPosePath.val);
        }

        public void SetSkinPath()
        {
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_skinPresetPath);
            _presetPath = _skinPresetPath;
            _onlyFav = _onlyFavSkin;
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetSkinPathAct()
        {
            _presetPath = _skinPresetPath;
            _onlyFav = _onlyFavSkin;
            LoadPresetsList(_savedSkinPath.val);
        }

        IEnumerator MoveRoot()
        {
            if (_min >= 0)
            {
                _min = 0;
            }
            else
            {
                if (_enablePositionAdjust.val)
                {
                    _min -= 0.1f;
                }
                else
                {
                    _min = 0;
                }
            }

            _targetAtom.GetStorableByID("control").transform.position = _personPosi - new Vector3(0, _min, 0);

            yield return new WaitForSeconds(0.5f);
            foreach (var item in _activeClothingList)
            {
                _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = true;
            }
            _targetAtom.tempDisableRender = !_targetAtom.tempDisableRender;
            _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = true;

            _targetAtom.GetStorableByID("control").transform.position = _personPosi;
            _targetAtom.GetStorableByID("control").transform.rotation = Quaternion.Euler(new Vector3(0, 0, 0));
            yield return new WaitForSeconds(0.5f);
            _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = false;
        }

        public void LoadPresetsList(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(_presetPath) && path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    path = pathSplit[0] + ":/" + _presetPath;
                }

                _varFileName = "";
                string favPath;
                if (path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    _varFullPath = pathSplit[0];
                    var varNameSplit = _varFullPath.Split('/');
                    var varNumber = varNameSplit.Length;
                    _varFullName = varNameSplit[varNumber - 1];
                    _varFileName = _varFullName.Remove(_varFullName.Length - 4, 4);
                    favPath = _filePrefs + _varFileName + pathSplit[1];
                }
                else
                {
                    favPath = path;
                }

                string[] presetFiles;
                string[] favFiles;
                string presetFileList = "";
                string favFileList = "";

                if (_enableSubfolder.val)
                {
                    ArrayList allFiles = new ArrayList();
                    GetAllFiles(path, "Preset_*.vap", ref allFiles);
                    presetFiles = (string[])allFiles.ToArray(typeof(string));
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    allFiles = new ArrayList();
                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        GetAllFiles(favPath, "Preset_*.vap.fav", ref allFiles);
                        favFiles = (string[])allFiles.ToArray(typeof(string));
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }
                else
                {
                    presetFiles = FileManagerSecure.GetFiles(path, "Preset_*.vap");
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        favFiles = FileManagerSecure.GetFiles(favPath, "Preset_*.vap.fav");
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }

                if (_presetPath == _clothingPresetPath)
                {
                    _presetClothingList = new List<string>(_presetList);
                    _presetClothingPath = presetFileList;
                    _favClothingList = new List<string>(_favList);
                    _favClothingPath = favFileList;

                    if (_onlyFavClothing.val)
                    {
                        _clothingFilesInFolder.val = _favClothingPath;
                        _clothingList = new List<string>(_favClothingList);
                    }
                    else
                    {
                        _clothingFilesInFolder.val = _presetClothingPath;
                        _clothingList = new List<string>(_presetClothingList);
                    }
                    _savedClothingPath.val = path;
                    _currentClothing.val = "";
                    _enableClothing.val = true;
                }
                else if (_presetPath == _hairPresetPath)
                {
                    _presetHairList = new List<string>(_presetList);
                    _presetHairPath = presetFileList;
                    _favHairList = new List<string>(_favList);
                    _favHairPath = favFileList;

                    if (_onlyFavHair.val)
                    {
                        _hairFilesInFolder.val = _favHairPath;
                        _hairList = new List<string>(_favHairList);
                    }
                    else
                    {
                        _hairFilesInFolder.val = _presetHairPath;
                        _hairList = new List<string>(_presetHairList);
                    }
                    _savedHairPath.val = path;
                    _currentHair.val = "";
                    _enableHair.val = true;
                }
                else if (_presetPath == _posePresetPath)
                {
                    _presetPoseList = new List<string>(_presetList);
                    _presetPosePath = presetFileList;
                    _favPoseList = new List<string>(_favList);
                    _favPosePath = favFileList;

                    if (_onlyFavPose.val)
                    {
                        _poseFilesInFolder.val = _favPosePath;
                        _poseList = new List<string>(_favPoseList);
                    }
                    else
                    {
                        _poseFilesInFolder.val = _presetPosePath;
                        _poseList = new List<string>(_presetPoseList);
                    }
                    _savedPosePath.val = path;
                    _currentPose.val = "";
                    _enablePose.val = true;
                }
                else
                {
                    _presetSkinList = new List<string>(_presetList);
                    _presetSkinPath = presetFileList;
                    _favSkinList = new List<string>(_favList);
                    _favSkinPath = favFileList;

                    if (_onlyFavSkin.val)
                    {
                        _skinFilesInFolder.val = _favSkinPath;
                        _skinList = new List<string>(_favSkinList);
                    }
                    else
                    {
                        _skinFilesInFolder.val = _presetSkinPath;
                        _skinList = new List<string>(_presetSkinList);
                    }
                    _savedSkinPath.val = path;
                    _currentSkin.val = "";
                    _enableSkin.val = true;
                }
            }
        }

        public static void GetAllFiles(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                GetAllFiles(subF, filter, ref allFiles);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                if (_savedClothingPath != null && _savedClothingPath.val != "")
                {
                    _presetPath = _clothingPresetPath;
                    _clothingList = new List<string>();
                    LoadPresetsList(_savedClothingPath.val);
                }

                if (_savedHairPath != null && _savedHairPath.val != "")
                {
                    _presetPath = _hairPresetPath;
                    _hairList = new List<string>();
                    LoadPresetsList(_savedHairPath.val);
                }

                if (_savedPosePath != null && _savedPosePath.val != "")
                {
                    _presetPath = _posePresetPath;
                    _poseList = new List<string>();
                    LoadPresetsList(_savedPosePath.val);
                }

                if (_savedSkinPath != null && _savedSkinPath.val != "")
                {
                    _presetPath = _skinPresetPath;
                    _skinList = new List<string>();
                    LoadPresetsList(_savedSkinPath.val);
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                if (_previousSelectCount != _selectCount.val)
                {
                    _waitCount = 0;
                }

                if (!_enableClothing.val)
                {
                    _previousClothingNumber = 0;
                }

                if (!_enableHair.val)
                {
                    _previousHairNumber = 0;
                }

                if (!_enablePose.val)
                {
                    _previousPoseNumber = 0;
                }

                if (!_enableSkin.val)
                {
                    _previousSkinNumber = 0;
                }

                _selectTime = int.Parse(_selectCount.val);
                if (_enableAutoLoad.val && _loopStart)
                {
                    if (_waitCount <= 0)
                    {
                        _waitCount = _selectTime;
                        _startTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        _enablePush = true;
                        LoadPreset();
                        _currentTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        if (_selectTime < _currentTime - _startTime)
                        {
                            SuperController.LogMessage("Please select interval time more than " + (_currentTime - _startTime + 1).ToString());
                            _selectCount.val = "30";
                            _selectTime = int.Parse(_selectCount.val);
                        }
                    }
                    _currentTime = DateTime.Now.DayOfYear * 24 * 60 * 60 + DateTime.Now.Hour * 60 * 60 + DateTime.Now.Minute * 60 + DateTime.Now.Second;
                    _waitCount = _selectTime - (_currentTime - _startTime);
                }
                else
                {
                    _loopStart = false;
                    _waitCount = _selectTime;
                }

                _previousSelectCount = _selectCount.val;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {

        }
    }
}